/*
 * This class file belongs to the IntoJ project. See enclosed license file for
 * licensing issues.
 * 
 * For more information, please visit http://www.intoj.org/
 *  
 */
package org.intoJ.inferType.exception;

/**
 *
 * Exceptions of this type are thrown when something goes wrong during the
 * calculation process.
 *
 * 
 * @version: $Date: 2007/07/28 12:12:55 $Author: kegel $Revision: 1.1 $
 * @author Philip Mayer
 *  
 */
public class IntoJCalculationException extends InterruptedException {

	private static final long serialVersionUID = -8162721171683494183L;

	private String title;
	private Throwable throwable;

	/**
	 * Constructor for IntoJCalculationException.
	 * @param title String
	 * @param message String
	 */
	public IntoJCalculationException(String title, String message) {

		super(message);
		this.title = title;
	}

	/**
	 * Constructor for IntoJCalculationException.
	 * @param title String
	 * @param message String
	 * @param e Throwable
	 */
	public IntoJCalculationException(String title, String message, Throwable e) {

		super(message);
		this.throwable = e;
		this.title = title;
	}

	/**
	 * Returns the title
	 * @return String
	 */
	public String getTitle() {

		return title;
	}

	/**
	 * Returns the cause.
	 * @return Throwable
	 */
	public Throwable getCause() {

		return throwable;
	}

}