/*
 * This class file belongs to the IntoJ project. See enclosed license file for
 * licensing issues.
 * 
 * For more information, please visit http://www.intoj.org/
 *  
 */
package org.intoJ.inferType.model.collection;

import java.util.Iterator;

import org.intoJ.inferType.model.model.DeclarationElement;

/**
 * 
 * This set contains DeclarationElements, like variables or methods. Each
 * element inside is unique (as the name "Set" implies) - according to the
 * signature of the declaration element.
 * 
 * @see DeclarationElement.equalsSignature()
 * 
 * @version: $Date: 2007/07/28 12:12:55 $Author: kegel $Revision: 1.1 $
 * @author Philip Mayer
 */
public class DESet_Signature extends DESet_Key {

	/*
	 * Creates a new set.
	 */
	public DESet_Signature() {

		super();
	}

	/**
	 * Convenience constructor for converting Key Lists into Signature lists
	 * 
	 * @param collectionOfDeclarationElements
	 */
	public DESet_Signature(DESet_Key collectionOfDeclarationElements) {

		super();
		addDeclarations(collectionOfDeclarationElements);
	}

	/**
	 * Adds a declaration.
	 * 
	 * @param element DeclarationElement
	 */
	public void addDeclaration(DeclarationElement element) {

		for (Iterator i = actualSet.iterator(); i.hasNext();) {
			DeclarationElement de = (DeclarationElement) i.next();
			if (de.equalsSignature(element))
				return;
		}
		actualSet.add(element); // Set already checks whether element is already
		// inside using equals
	}

	/**
	 * Adds several declarations.
	 * 
	 * @param collectionOfDeclarationElements DESet_Key
	 */
	public void addDeclarations(DESet_Key collectionOfDeclarationElements) {

		for (Iterator i = collectionOfDeclarationElements.deIterator(); i.hasNext();) {
			addDeclaration((DeclarationElement) i.next());
		}
	}

	/**
	 * Checks according to signature.
	 * 
	 * @param someInvocation
	 * @return boolean
	 */
	public boolean containsDeclaration(DeclarationElement someInvocation) {

		for (Iterator i = actualSet.iterator(); i.hasNext();) {
			DeclarationElement de = (DeclarationElement) i.next();
			if (de.equalsSignature(someInvocation))
				return true;
		}
		return false;
	}

	/**
	 * Checks if this set contains the given set.
	 * 
	 * @param elements
	 * @return boolean
	 */
	public boolean containsDeclarations(DESet_Key element) {

		for (Iterator i = element.deIterator(); i.hasNext();) {
			DeclarationElement de = (DeclarationElement) i.next();
			if (!containsDeclaration(de))
				return false;
		}
		return true;
	}

	public void removeDeclarations(DESet_Key elements) {

		for (Iterator i = deIterator(); i.hasNext();) {
			DeclarationElement de = (DeclarationElement) i.next();
			if (elements.containsDeclaration(de))
				i.remove();
		}
	}

	/**
	 * Returns a new set which is the union (based on signature) of this and the other set.
	 * This operation does not change the set. 
	 * 
	 * @param other
	 * @return DESet_Signature
	 */
	public DESet_Signature union(DESet_Signature other) {

		DESet_Signature tmp = new DESet_Signature();
		tmp.addDeclarations(this);
		tmp.addDeclarations(other);
		return tmp;
	}

	/**
	 * Returns the intersection (based on signature) of this and the other set.
	 * This operation does not change the set.
	 * 
	 * @param other
	 * @return DESet_Signature
	 */
	public DESet_Signature intersection(DESet_Signature other) {

		DESet_Signature tmp = new DESet_Signature();
		for (Iterator i = other.deIterator(); i.hasNext();) {
			DeclarationElement de = (DeclarationElement) i.next();
			if (containsDeclaration(de))
				tmp.addDeclaration(de);
		}
		return tmp;
	}

}