/*
 * This class file belongs to the IntoJ project. See enclosed license file for
 * licensing issues.
 * 
 * For more information, please visit http://www.intoj.org/
 *  
 */
package org.intoJ.inferType3.multirefac;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.intoJ.inferType.exception.UserAbortedException;

/**
 * This action is instantiated every time the user selects the "Infer Type Multi Refactoring..." 
 * entry in a project's popup menu.
 * 
 * $Date: 2007/07/28 12:12:55 $Author: kegel $Revision: 1.1 $
 * @author Philip Mayer
 */
public class RunMultiRefacAction implements IActionDelegate {

	private ISelection selection;
	private MultiRefactorer multirefac;

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction, org.eclipse.ui.IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void run(IAction action) {

		if (!(selection instanceof IStructuredSelection))
			return;

		// There is only one selection (see plugin.xml)
		IStructuredSelection structured = (IStructuredSelection) selection;
		final IJavaElement javaElement = (IJavaElement) structured.getFirstElement();
		runWithRunnable(javaElement);
	}

	/**
	 * 
	 * Returns the current shell
	 * 
	 * @return shell
	 */
	protected Shell getShell() {

		IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		if (window == null)
			return null;
		return window.getShell();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {

		this.selection = selection;
	}

	/* (non-Javadoc)
	 * @see org.intoJ.inferType.evaluation.actions.AbstractAction#runWithRunnable(org.eclipse.jdt.core.IJavaElement)
	 */
	protected void runWithRunnable(final IJavaElement javaElement) {

		IRunnableWithProgress op = new IRunnableWithProgress() {

			public void run(IProgressMonitor monitor) throws InterruptedException {

				multirefac.run(javaElement, monitor);
			}
		};
		
        StartDialog startDialog = new StartDialog(getShell());
        if (!startDialog.checkOpen()) {
			return;
        }
		

		multirefac = new MultiRefactorer(startDialog.isChangeTypeArgs(), startDialog.isNoAbstractClasses());
		ProgressMonitorDialog dialog = new ProgressMonitorDialog(getShell());

		try {
			dialog.run(true, true, op);

			String message = multirefac.getErrorMessage();
			boolean error = multirefac.isError();
			if ((message == null) || (message.equals(""))) {
				error = true;
				message = "An Exception occurred. Please see the Error Log.";
			}

			if (error)
				MessageDialog.openError(getShell(), "Infer Type was not executable", message);
			else
				MessageDialog.openInformation(getShell(), "Infer Type", message);

		} catch (InvocationTargetException e) {
			ILog.logError("InvocationTargetException in Multi Refactorer", e);
        } catch (UserAbortedException e) {
            //ok
		} catch (InterruptedException e) {
            ILog.logError("InterruptedException in Multi Refactorer", e);
		} catch (Exception e) {
            ILog.logError("Exception in Multi Refactorer", e);
		}
	}
}
