/*
 * This file belongs to the IntoJ project. See enclosed license file for
 * licensing issues.
 * 
 * For more information, please visit http://www.intoj.org/
 *  
 */
package org.intoJ.inferType3.internal.actions;

import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IImportDeclaration;
import org.eclipse.jdt.core.IMember;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.dom.ITypeBinding;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.intoJ.inferType3.internal.constraintModel.util.Scope;
import org.intoJ.inferType3.internal.constraintModel.util.SelectedElement;
import org.intoJ.inferType3.internal.ui.ImportScopeDialog;
import org.intoJ.inferType3.internal.util.ConversionUtil;
import org.intoJ.inferType3.internal.util.InternalCalculationException;

/**
 * This action can be executed by the user by selecting "Infer Type" from the
 * context menus of fields, methods and imports.
 * 
 * @author Andreas Mei&szlig;ner
 * @author Hannes Kegel
 */
public class InferTypeOnMemberAction extends AbstractInferTypeAction implements IObjectActionDelegate {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction,
	 *      org.eclipse.ui.IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void run(IAction action) {
		if (!(selection instanceof IStructuredSelection)) {
			return;
        }

		IStructuredSelection structured = (IStructuredSelection) selection;
    
        SelectedElement selectedElement = null;
		try {
            // Could be invoked anywhere (per key binding)
            Object o = structured.getFirstElement();
            if (o instanceof IMember) {
                IMember member = (IMember)o;
        
                if (!(member instanceof IMethod) && !(member instanceof IField)) {
                    return;
                }

                ConversionUtil.Result result = ConversionUtil.findBinding(member);
    			selectedElement = new SelectedElement(result.getBinding(), result.getTypeStartPos(), null);
            } else if (o instanceof IImportDeclaration) {
                IImportDeclaration importDeclaration = (IImportDeclaration)o;
                
                String typeName = importDeclaration.getElementName();
                if (typeName.endsWith("*")) {
                    showError("Import Declaration", "Refactoring of on-demand imports is not supported.");
                    return;                                    
                }
                IType type = importDeclaration.getJavaProject().findType(typeName);
                if (type == null) {
                    showError("Import Declaration", "Could not find type " + typeName + ".");
                    return;                                                        
                }                
                ITypeBinding binding = ConversionUtil.getTypeBinding(type);
                
                Scope scope = ImportScopeDialog.askForScope(getShell());
                if (scope == null) {
                    return;
                }
                
                selectedElement = new SelectedElement(binding, importDeclaration, scope);                
            }

            if (selectedElement == null) {
                return;
            }
            
			runWithSelectedElement(selectedElement);

		} catch (InternalCalculationException e) {
			MessageDialog.openError(getShell(), "Infer Type Error", e.getMessage());
		} catch (InterruptedException e) {
            e.printStackTrace();
        } catch (JavaModelException e) {
            MessageDialog.openError(getShell(), "Infer Type Error", e.getMessage());
        }
	}
}