/*
 * This file belongs to the IntoJ project. See enclosed license file for
 * licensing issues.
 * 
 * For more information, please visit http://www.intoj.org/
 *  
 */
package org.intoJ.inferType3.internal.constraintModel.util;

import org.eclipse.jdt.core.dom.IMethodBinding;
import org.eclipse.jdt.core.dom.ITypeBinding;

/**
 * 
 * Find constructor of anonymous class creation
 * 
 * Ripped out of JDT:
 * 
 * org.eclipse.jdt.internal.corext.refactoring.code.ConvertAnonymousToNestedRefactoring
 * 
 * @author Philip Mayer
 *
 */
public class AnonWorkaround {

	public static IMethodBinding getSuperConstructorBinding(IMethodBinding anonConstr) {

		//workaround for missing java core functionality - finding a
		// super constructor for an anonymous class creation
		if (anonConstr == null) {
            return null;
        }
		ITypeBinding superClass = anonConstr.getDeclaringClass().getSuperclass();
		IMethodBinding[] superMethods = superClass.getDeclaredMethods();
		for (IMethodBinding superMethod : superMethods) {
			if (superMethod.isConstructor() && parameterTypesMatch(superMethod, anonConstr)) {
                return superMethod;
            }
		}
		//Assert.isTrue(false);//there's no way - it must be there
		return null;
	}

	private static boolean parameterTypesMatch(IMethodBinding m1, IMethodBinding m2) {

		ITypeBinding[] m1Params = m1.getParameterTypes();
		ITypeBinding[] m2Params = m2.getParameterTypes();
		if (m1Params.length != m2Params.length) {
            return false;
        }
		for (int i = 0; i < m2Params.length; i++) {
			if (!m1Params[i].equals(m2Params[i])) {
                return false;
            }
		}
		return true;
	}

}
