/*
 * This file belongs to the IntoJ project. See enclosed license file for
 * licensing issues.
 * 
 * For more information, please visit http://www.intoj.org/
 *  
 */
package org.intoJ.inferType3.internal.constraintModel.variables;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.dom.IMethodBinding;
import org.eclipse.jdt.internal.corext.refactoring.typeconstraints.types.TType;
import org.eclipse.jdt.internal.corext.refactoring.typeconstraints2.ConstraintVariable2;
import org.eclipse.jdt.internal.corext.refactoring.typeconstraints2.ISourceConstraintVariable;
import org.eclipse.jdt.internal.corext.refactoring.typeconstraints2.ReturnTypeVariable2;

/**
 * An IndependentReturnTypeVariable stands for the return type of a method, but is independent
 * from other invocations or from the declaration. 
 * <p>
 * In constrast to {@link ReturnTypeVariable2} there is not only one distinct variable 
 * per method. This independent variable is needed to connect return types to receiver elements when 
 * the return type contains type variables or wildcards, but avoids connecting all invocations 
 * independent of the receiver.
 * </p> 
 * @author Hannes Kegel
 *
 */
public class IndependentReturnTypeVariable extends ConstraintVariable2 implements ISourceConstraintVariable {

    private final String fKey;
    private ICompilationUnit fCompilationUnit;

    /**
     * The single constructor.
     * 
     * @param type the current return type
     * @param binding the method invoked
     */
    public IndependentReturnTypeVariable(TType type, IMethodBinding binding) {
        super(type);
        fKey= binding.getKey();
    }

    /* (non-Javadoc)
     * @see org.eclipse.jdt.internal.corext.refactoring.typeconstraints2.ISourceConstraintVariable#setCompilationUnit(org.eclipse.jdt.core.ICompilationUnit)
     */
    public void setCompilationUnit(ICompilationUnit unit) {
        fCompilationUnit= unit;
    }

    /* (non-Javadoc)
     * @see org.eclipse.jdt.internal.corext.refactoring.typeconstraints2.ISourceConstraintVariable#getCompilationUnit()
     */
    public ICompilationUnit getCompilationUnit() {
        return fCompilationUnit;
    }

    /* (non-Javadoc)
     * @see org.eclipse.jdt.internal.corext.refactoring.typeconstraints2.ConstraintVariable2#toString()
     */
    @Override
    public String toString() {
        String toString= (String) getData(TO_STRING);
        return toString == null ? "[IndependentReturnType(" + fKey + ")]" : toString; 
    }
}
