/*
 * This file belongs to the IntoJ project. See enclosed license file for
 * licensing issues.
 * 
 * For more information, please visit http://www.intoj.org/
 *  
 */
package org.intoJ.inferType3.internal.util;

import java.util.AbstractSet;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.jdt.internal.corext.refactoring.typeconstraints2.ConstraintVariable2;

/** 
 * Customized implementation of a hash set used to store instances of
 * {@link ConstraintVariable2}.
 * The reason for this implementation is the addOrGet method that prefers
 * existing equal variables.
 *
 * @author Hannes Kegel
 */
public class VariableSet extends AbstractSet<ConstraintVariable2> {

    /** The backing hash map */
    private final Map<ConstraintVariable2, ConstraintVariable2> store = new HashMap<ConstraintVariable2, ConstraintVariable2>();

    /**
     * Returns an existing equal variable if possible, 
     * otherwise adds the specified variable to this set.
     * 
     * @param object the variable to add
     * @return an already existing variable considered equal to the specified
     *         one, or the newly added variable
     */
    public final ConstraintVariable2 addOrGet(final ConstraintVariable2 object) {
        final ConstraintVariable2 result = store.get(object);
        if (result == null) {
            add(object);
            return object;
        } else {
            return result;
        }
    }

    /*
     * @see java.util.AbstractCollection#add(java.lang.Object)
     */
    @Override
    public final boolean add(final ConstraintVariable2 object) {
        return store.put(object, object) == null;
    }

    /*
     * @see java.util.AbstractCollection#clear()
     */
    @Override
    public final void clear() {
        store.clear();
    }

    /*
     * @see java.util.AbstractCollection#contains(java.lang.Object)
     */
    @Override
    public final boolean contains(final Object object) {
        return store.containsKey(object);
    }

    /*
     * @see java.util.AbstractCollection#isEmpty()
     */
    @Override
    public final boolean isEmpty() {
        return store.isEmpty();
    }

    /*
     * @see java.util.AbstractCollection#iterator()
     */
    @Override
    public final Iterator<ConstraintVariable2> iterator() {
        return store.keySet().iterator();
    }

    /*
     * @see java.util.AbstractCollection#remove(java.lang.Object)
     */
    @Override
    public final boolean remove(final Object object) {
        return store.remove(object) == object;
    }

    /*
     * @see java.util.AbstractCollection#size()
     */
    @Override
    public final int size() {
        return store.size();
    }
}
