/*
 * This class file belongs to the IntoJ project. See enclosed license file for
 * licensing issues.
 * 
 * For more information, please visit http://www.intoj.org/
 *  
 */
package org.intoJ.inferType.model.collection;

import java.util.Iterator;

import org.intoJ.inferType.model.model.TypeElement;

/**
 * 
 * This class represents a type hierarchy, i..e. a set of types (TypeElements) which are part of
 * a hierarchy. The root type is the starting point. 
 * 
 * @version: $Date: 2007/09/09 19:40:25 $Author: kegel $Revision: 1.1 $
 *  
 * @author Philip Mayer
 */
public class HierarchySet {

	private TypeElement root;
	private TypeSet allTypes;

	/**
	 * Constructor for HierarchySet.
	 * 
	 * @param root TypeElement
	 * @param allTypes TypeSet
	 */
	public HierarchySet(TypeElement root, TypeSet allTypes) {

		this.root = root;
		this.allTypes = allTypes;
	}

	/**
	 * Returns the root of the hierarchy.
	 * 
	 * @return TypeElement
	 */
	public TypeElement getRoot() {

		return root;
	}

	/**
	 * Returns all types of the hierarchy (including root).
	 * @return TypeSet
	 */
	public TypeSet getAllTypes() {

		return allTypes;
	}

	/**
	 * Returns all types of the hierarchy (excluding root).
	 * @return TypeSet
	 */
	public TypeSet getSuperTypes() {

		TypeSet set = new TypeSet();
		set.addTypes(allTypes);
		set.removeType(root);
		return set;
	}

	/**
	 * Returns all subtypes of the given type in the hierarchy. 
	 * 
	 * @param type TypeElement
	 * @return TypeSet
	 */
	public TypeSet getSubTypesOfType(TypeElement type) {

		TypeSet subtypes = new TypeSet();

		if (root.getAllSuperTypes().contains(type))
			subtypes.addType(type);

		for (Iterator iter = allTypes.typeIterator(); iter.hasNext();) {
			TypeElement currentType = (TypeElement) iter.next();
			if (currentType.getAllSuperTypes().contains(type))
				subtypes.addType(currentType);
		}

		subtypes.removeType(type);

		return subtypes;
	}

	public String toString() {

		StringBuffer b = new StringBuffer();
		for (Iterator i = allTypes.typeIterator(); i.hasNext();) {
			TypeElement e = (TypeElement) i.next();
			b.append(e);
			b.append(";");
		}
		return b.toString();

	}

}