/*
 * This file belongs to the IntoJ project. See enclosed license file for
 * licensing issues.
 * 
 * For more information, please visit http://www.intoj.org/
 *  
 */
package org.intoj.riwd.ui;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.intoJ.inferType3.internal.typeCalc.hierarchy.BindingWrapper;
import org.intoJ.inferType3.internal.typeCalc.hierarchy.ExistingTCType;
import org.intoJ.inferType3.internal.typeCalc.hierarchy.TCType;
import org.intoj.riwd.RiwdRefactoring;

/**
 * The check state listener used for the tree in the Infer Type
 * wizard. 
 * 
 * @author Hannes Kegel
 *
 */
class CheckStateListener implements ICheckStateListener {
    private RiwdRefactoring refactoring;
    
    /**
     * The single constructor
     * @param wizard the infer type wizard
     */
    public CheckStateListener(RiwdRefactoring refactoring) {
        this.refactoring = refactoring;
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.viewers.ICheckStateListener#checkStateChanged(org.eclipse.jface.viewers.CheckStateChangedEvent)
     */
    public void checkStateChanged(CheckStateChangedEvent event) {
        Object element = event.getElement();

        CheckboxTreeViewer checkBoxTreeViewer = (CheckboxTreeViewer)event.getCheckable();
        ITContentProvider contentProvider = (ITContentProvider)checkBoxTreeViewer.getContentProvider();

        if (element instanceof BindingWrapper) {
            checkBoxTreeViewer.setChecked(element, true);
        } else if (element instanceof TCType) {
            checkBoxTreeViewer.setSubtreeChecked(element, event.getChecked());
            checkBoxTreeViewer.setGrayed(element, false);                                        
        } else {
            checkBoxTreeViewer.setChecked(element, event.getChecked());
            
            Object parent = contentProvider.getParent(element);
            while (parent != null) {
                boolean allChecked = true;
                boolean noneChecked = true;
                
                for (Object child : contentProvider.getChildren(parent)) {
                    if (checkBoxTreeViewer.getChecked(child)) {
                        noneChecked = false;
                    } else {
                        allChecked = false;
                    }
                }
                if (noneChecked) {
                    checkBoxTreeViewer.setChecked(parent, false);
                    checkBoxTreeViewer.setGrayed(parent, false);                                        
                } else if (allChecked) {
                    checkBoxTreeViewer.setChecked(parent, true);                                        
                    checkBoxTreeViewer.setGrayed(parent, false);                                        
                } else {
                    checkBoxTreeViewer.setGrayChecked(parent, true);                                        
                }
                
                parent = contentProvider.getParent(parent);
            }
        }
        List<ExistingTCType> checkedInterfaces = new ArrayList<ExistingTCType>();
        List<BindingWrapper> checkedWrapper = new ArrayList<BindingWrapper>();
        for (Object selectedElement : checkBoxTreeViewer.getCheckedElements()) {
            if (selectedElement instanceof WrapperLeaf) {
                checkedWrapper.add(((WrapperLeaf)selectedElement).getWrapper());
            } else if (selectedElement instanceof ExistingTCType && !checkBoxTreeViewer.getGrayed(selectedElement)) {
                ExistingTCType existingTCType = (ExistingTCType)selectedElement;
                if (existingTCType.isInterface())  {
                    checkedInterfaces.add(existingTCType);
                }
            }
        }
        refactoring.setCheckedMembers(checkedWrapper, checkedInterfaces);
    }
    
}    

