/*
 * This file belongs to the IntoJ project. See enclosed license file for
 * licensing issues.
 * 
 * For more information, please visit http://www.intoj.org/
 *  
 */
package org.intoj.riwd.ui;

import org.eclipse.jdt.internal.ui.viewsupport.BindingLabelProvider;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.swt.graphics.Image;
import org.intoJ.inferType3.internal.typeCalc.hierarchy.BindingWrapper;
import org.intoJ.inferType3.internal.typeCalc.hierarchy.ExistingTCType;
import org.intoJ.inferType3.internal.ui.InferTypeWizard;

/**
 * The label provider for the tree in the Infer Type wizard.
 * 
 * @author Hannes Kegel
 *
 */
class ITLabelProvider extends BindingLabelProvider {
    private static final Image ACCESS_SET_METHOD_IMAGE = ImageDescriptor.createFromURL(
            InferTypeWizard.class.getResource("images/arrow_right.gif")).createImage();

    /* (non-Javadoc)
     * @see org.eclipse.jdt.internal.ui.viewsupport.BindingLabelProvider#getImage(java.lang.Object)
     */
    @Override
    public Image getImage(Object element) {
        if (element instanceof BindingWrapper) {
            return ACCESS_SET_METHOD_IMAGE;
        } else if (element instanceof ExistingTCType) {
            return super.getImage(((ExistingTCType)element).getBaseTypeBinding());
        } else if (element instanceof WrapperLeaf) {                    
            return super.getImage(((WrapperLeaf)element).getWrapper().getMethodBinding());
        }
        return null;
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.jdt.internal.ui.viewsupport.BindingLabelProvider#getText(java.lang.Object)
     */
    @Override
    public String getText(Object element) {
        if (element instanceof BindingWrapper) {
            BindingWrapper wrapper = (BindingWrapper)element;
            return wrapper.toString() + " (declared in " + wrapper.getMethodBinding().getDeclaringClass().getQualifiedName() + ")";
        }
        return element.toString();
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.viewers.BaseLabelProvider#addListener(org.eclipse.jface.viewers.ILabelProviderListener)
     */
    @Override
    public void addListener(ILabelProviderListener listener) {
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.viewers.BaseLabelProvider#dispose()
     */
    @Override
    public void dispose() {
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.viewers.BaseLabelProvider#isLabelProperty(java.lang.Object, java.lang.String)
     */
    @Override
    public boolean isLabelProperty(Object element, String property) {
        return false;
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.viewers.BaseLabelProvider#removeListener(org.eclipse.jface.viewers.ILabelProviderListener)
     */
    @Override
    public void removeListener(ILabelProviderListener listener) {
    }
    
}
